
function logger(log) {
    console.log(log)
}

// Start waiting process
function startFetch() {
    document.getElementById("fetch").style.display = 'inline';
    logger('Start Fetch Called')
}

// Stop waiting process
function stopFetch() {
    document.getElementById("fetch").style.display = 'none';
    logger('Stop Fetch Called')
}

// Plugin initialization process
function initializePlugin() {
    iframe = document.createElement('iframe');
    iframe.src = getEnvironment()["iframeSrc"]
    document.body.appendChild(iframe);
    document.getElementById('version').innerHTML = 'Version ' + getEnvironment()["version"];
}

// Show pliugin warnings
function showWarning(warningText){
    logger(warningText)
    if(window.webkit && window.webkit.messageHandlers.sketchPlugin){
		window.webkit.messageHandlers.sketchPlugin.postMessage(JSON.stringify({type: 'error', content: getLang()[warningText]}));
	} else {
		console.error("Failed to send options - could not to find 'sketchPlugin' message handler. Is every thing set up properly for messaging?");
	}
    stopFetch()
}

// Transform buffer to base64
function arrayBufferToBase64Encoder(arrayBuffer) {
    var base64String = '';
    var bytesString = new Uint8Array(arrayBuffer);
    var len = bytesString.byteLength;
    for (var i = 0; i < len; i++) {
        base64String += String.fromCharCode(bytesString[i] );
    }
    return btoa(base64String);
}

// Post image Base64 content to service
function postImageContent(imageBase64, imageType, width, height){
    logger(`postImageContent called in UI`)

    var base64Content = arrayBufferToBase64Encoder(imageBase64)


    if(window.webkit && window.webkit.messageHandlers.sketchPlugin){
		window.webkit.messageHandlers.sketchPlugin.postMessage(JSON.stringify({type: 'imageContent', content: base64Content, width: width, height: height}));
	} else {
		console.error("Failed to send options - could not to find 'sketchPlugin' message handler. Is every thing set up properly for messaging?");
	}

    stopFetch()
    
}

// Downloading image by url, resize and insert after
function getImageByUrl(url, width, height, imageType) {
    logger(`getImageByUrl called in UI`)
    var stubUrl = getEnvironment()["stubUrl"]
    var apiUrl = `${stubUrl}/${url}`;

    if (width === undefined || height === undefined) {
        var newImage = new Image()
        newImage.crossOrigin="anonymous";
        newImage.src = apiUrl
        newImage.onload = function() {
            width = newImage.width
            height = newImage.height
        }
        newImage.onerror = function() {
            logger(`Image type was not defined, it is not an image!`)
            showWarning('typeError')
        }
    }

    // Insert image without content resize
    fetch(apiUrl).then(response => {
        data = response.arrayBuffer()
        data.then(imageContent => {
            try {
                postImageContent(new Uint8Array(imageContent), imageType, width, height)
            }
            catch {
                showWarning("insertError")
            }
        })
    }).catch(error => {showWarning("downloadError")}) 
}

// Parse incoming messages process. If message includes "resource_download" parameter, insert image process starts
function parseMessage(data) {
    if (typeof data === "string") 
        data = JSON.parse(data)
    
    // Supported Figma MIME types for inserting images
    var allowedMIMES = getEnvironment()["allowedMIMES"]
    var stubUrl = getEnvironment()["stubUrl"]
    if (data.action === 'resource_download' && data !== undefined && data !== '') {
        const downloadUrl = data.download_url;
        var fileSize = parseInt(data.filesize)
        var width = parseInt(data.width)
        var height = parseInt(data.height)
        var imageType = data.type

        if (isNaN(width) || isNaN(height) || width == 0 || height == 0 || downloadUrl.indexOf('&preset=') > -1) {
            width = undefined
            height = undefined
        }
        if (allowedMIMES.indexOf(data.type) !== -1 && downloadUrl.indexOf('&preset=') == -1){
            startFetch()
            getImageByUrl(downloadUrl, width, height, imageType, fileSize)
        }
        else if (downloadUrl.indexOf('&preset=') > -1) {
            startFetch()
            const apiUrl = `${stubUrl}/${downloadUrl}`;
            var newImage = new Image()
            newImage.crossOrigin="anonymous";
            newImage.src = apiUrl
            newImage.onload = function() {
                logger(`Image type was not defined, but it is an image!`)
                width = newImage.width
                height = newImage.height
                getImageByUrl(downloadUrl, width, height, imageType, fileSize)
            }
            newImage.onerror = function() {
                logger(`Image type was not defined, it is not an image!`)
                showWarning('typeError')
            }
        }
        else {
            showWarning('typeError')
        }
    }
}

// Incoming messages listener
onmessage = (event) => {
    logger(event.data)
    parseMessage(event.data)
}

document.addEventListener("DOMContentLoaded", () => {
	initializePlugin()
});